"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./src/config");
const axios_1 = __importDefault(require("axios"));
const querystring_1 = __importDefault(require("querystring"));
let config;
let oldMatches;
let oldParticipantNameMap = new Map();
let oldParticipantIdMap = new Map();
let newMatches;
let newParticipantNameMap = new Map();
let newParticipantIdMap = new Map();
async function getTournamentData(tournamentUrl) {
    const { data: { tournament } } = await axios_1.default.get(` https://api.challonge.com/v1/tournaments/${tournamentUrl}.json`, {
        responseType: "json",
        paramsSerializer: querystring_1.default.stringify,
        params: {
            api_key: config.apiKey,
            include_participants: 1,
            include_matches: 1
        },
    });
    const participants = tournament.participants.map(pd => pd.participant);
    const matches = tournament.matches.map(pd => pd.match);
    return { participants, matches };
}
async function loadOldTournamentData() {
    const { participants, matches } = await getTournamentData(config.sourceTournament);
    oldParticipantNameMap.clear();
    oldParticipantIdMap.clear();
    for (let participant of participants) {
        //console.log(`Inserting old participant ${participant.id} ${participant.name}`);
        oldParticipantNameMap.set(participant.name, participant);
        oldParticipantIdMap.set(participant.id, participant);
    }
    oldMatches = matches;
}
async function loadNewTournamentData() {
    const { participants, matches } = await getTournamentData(config.targetTournament);
    newParticipantIdMap.clear();
    newParticipantNameMap.clear();
    for (let participant of participants) {
        //console.log(`Inserting new participant ${participant.id} ${participant.name}`);
        newParticipantNameMap.set(participant.name, participant);
        newParticipantIdMap.set(participant.id, participant);
    }
    newMatches = matches;
}
async function checkQuit(round, oldPlayerId) {
    const oldPlayer = oldParticipantIdMap.get(oldPlayerId);
    if (oldPlayer.active) {
        return;
    }
    const nextRoundMatch = oldMatches.find(m => m.round === round + 1 && (m.player1_id === oldPlayerId || m.player2_id === oldPlayerId));
    if (!nextRoundMatch) {
        const newPlayerId = newParticipantNameMap.get(oldPlayer.name).id;
        console.log(`Player ${oldPlayer.name}: ${oldPlayerId}/${newPlayerId} quitted in round ${round}.`);
        while (true) {
            try {
                await axios_1.default.delete(`https://api.challonge.com/v1/tournaments/${config.targetTournament}/participants/${newPlayerId}.json?api_key=${encodeURIComponent(config.apiKey)}`, {
                    responseType: 'json'
                });
                console.log(`Successfully quitted player ${oldPlayer.name} in round ${round}.`);
                break;
            }
            catch (e) {
                console.log(`Failed quitting player ${oldPlayer.name} in round ${round}: ${e.toString()}`);
            }
        }
    }
}
async function migrateMatch(match) {
    const currentMatchPlayer1Name = newParticipantIdMap.get(match.player1_id).name;
    const currentMatchPlayer2Name = newParticipantIdMap.get(match.player2_id).name;
    const oldMatch = oldMatches.find(m => {
        if (m.round !== match.round) {
            return false;
        }
        const oldMatchPlayer1Name = oldParticipantIdMap.get(m.player1_id).name;
        const oldMatchPlayer2Name = oldParticipantIdMap.get(m.player2_id).name;
        return currentMatchPlayer1Name === oldMatchPlayer1Name && oldMatchPlayer2Name === currentMatchPlayer2Name;
    });
    if (!oldMatch) {
        console.error(`Match for ${currentMatchPlayer1Name} vs ${currentMatchPlayer2Name} not found.`);
        return;
    }
    let winner_id;
    if (typeof (oldMatch.winner_id) === "number" && oldParticipantIdMap.has(oldMatch.winner_id)) {
        const winnerName = oldParticipantIdMap.get(oldMatch.winner_id).name;
        winner_id = newParticipantNameMap.get(winnerName).id;
        console.log(`Migrating match ${currentMatchPlayer1Name} vs ${currentMatchPlayer2Name} with winner ${winnerName}.`);
    }
    else {
        winner_id = oldMatch.winner_id;
        if (oldMatch.scores_csv === "0-0") {
            winner_id = 'tie';
        }
        console.log(`Migrating match ${currentMatchPlayer1Name} vs ${currentMatchPlayer2Name} with result ${winner_id}.`);
    }
    const oldPlayerIds = [oldMatch.player1_id, oldMatch.player2_id];
    await Promise.all(oldPlayerIds.map(pid => checkQuit(match.round, pid)));
    while (true) {
        try {
            await axios_1.default.put(`https://api.challonge.com/v1/tournaments/${config.targetTournament}/matches/${match.id}.json`, {
                api_key: config.apiKey,
                match: {
                    scores_csv: oldMatch.scores_csv,
                    winner_id
                }
            }, config_1.axiosPostConfig);
            console.log(`Successfully migrated match ${currentMatchPlayer1Name} vs ${currentMatchPlayer2Name}.`);
            break;
        }
        catch (e) {
            console.error(`Failed migrating match ${currentMatchPlayer1Name} vs ${currentMatchPlayer2Name}: ${e.toString()}`);
        }
    }
}
async function main() {
    config = await (0, config_1.loadConfig)();
    console.log(`Reading old tournament ${config.sourceTournament}.`);
    await loadOldTournamentData();
    for (let round = 1; round <= config.maxRounds; ++round) {
        console.log(`Handling round ${round}.`);
        await loadNewTournamentData();
        const matches = newMatches.filter(m => m.round === round && m.state === "open");
        if (!matches.length) {
            console.log(`No available matches in round ${round}.`);
            continue;
        }
        await Promise.all(matches.map(match => migrateMatch(match)));
    }
    console.log(`Finished.`);
}
main();
//# sourceMappingURL=migrate.js.map