import {
  axiosPostConfig,
  Config,
  loadConfig,
  Match,
  MatchData,
  Participant,
  ParticipantData,
} from './src/config';
import axios from 'axios';
import qs from 'querystring';

let config: Config;
let oldMatches: Match[];
const oldParticipantNameMap: Map<string, Participant> = new Map();
const oldParticipantIdMap: Map<number, Participant> = new Map();
let newMatches: Match[];
const newParticipantNameMap: Map<string, Participant> = new Map();
const newParticipantIdMap: Map<number, Participant> = new Map();

async function getTournamentData(tournamentUrl: string) {
  const {
    data: { tournament },
  } = await axios.get(
    ` https://api.challonge.com/v1/tournaments/${tournamentUrl}.json`,
    {
      responseType: 'json',
      paramsSerializer: qs.stringify,
      params: {
        api_key: config.apiKey,
        include_participants: 1,
        include_matches: 1,
      },
    },
  );
  const participants = (tournament.participants as ParticipantData[]).map(
    (pd) => pd.participant,
  );
  const matches = (tournament.matches as MatchData[]).map((pd) => pd.match);
  return { participants, matches };
}

async function loadOldTournamentData() {
  const { participants, matches } = await getTournamentData(
    config.sourceTournament,
  );
  oldParticipantNameMap.clear();
  oldParticipantIdMap.clear();
  for (const participant of participants) {
    //console.log(`Inserting old participant ${participant.id} ${participant.name}`);
    oldParticipantNameMap.set(participant.name, participant);
    oldParticipantIdMap.set(participant.id, participant);
  }
  oldMatches = matches;
}

async function loadNewTournamentData() {
  const { participants, matches } = await getTournamentData(
    config.targetTournament,
  );
  newParticipantIdMap.clear();
  newParticipantNameMap.clear();
  for (const participant of participants) {
    //console.log(`Inserting new participant ${participant.id} ${participant.name}`);
    newParticipantNameMap.set(participant.name, participant);
    newParticipantIdMap.set(participant.id, participant);
  }
  newMatches = matches;
}

async function checkQuit(round: number, oldPlayerId: number) {
  const oldPlayer = oldParticipantIdMap.get(oldPlayerId);
  if (oldPlayer.active) {
    return;
  }
  const nextRoundMatch = oldMatches.find(
    (m) =>
      m.round === round + 1 &&
      (m.player1_id === oldPlayerId || m.player2_id === oldPlayerId),
  );
  if (!nextRoundMatch) {
    const newPlayerId = newParticipantNameMap.get(oldPlayer.name).id;
    console.log(
      `Player ${oldPlayer.name}: ${oldPlayerId}/${newPlayerId} quitted in round ${round}.`,
    );
    while (true) {
      try {
        await axios.delete(
          `https://api.challonge.com/v1/tournaments/${
            config.targetTournament
          }/participants/${newPlayerId}.json?api_key=${encodeURIComponent(
            config.apiKey,
          )}`,
          {
            responseType: 'json',
          },
        );
        console.log(
          `Successfully quitted player ${oldPlayer.name} in round ${round}.`,
        );
        break;
      } catch (e) {
        console.log(
          `Failed quitting player ${
            oldPlayer.name
          } in round ${round}: ${e.toString()}`,
        );
      }
    }
  }
}

async function migrateMatch(match: Match) {
  const currentMatchPlayer1Name = newParticipantIdMap.get(
    match.player1_id,
  ).name;
  const currentMatchPlayer2Name = newParticipantIdMap.get(
    match.player2_id,
  ).name;
  const oldMatch = oldMatches.find((m) => {
    if (m.round !== match.round) {
      return false;
    }
    const oldMatchPlayer1Name = oldParticipantIdMap.get(m.player1_id).name;
    const oldMatchPlayer2Name = oldParticipantIdMap.get(m.player2_id).name;
    return (
      currentMatchPlayer1Name === oldMatchPlayer1Name &&
      oldMatchPlayer2Name === currentMatchPlayer2Name
    );
  });
  if (!oldMatch) {
    console.error(
      `Match for ${currentMatchPlayer1Name} vs ${currentMatchPlayer2Name} not found.`,
    );
    return;
  }
  let winner_id: string | number;
  if (
    typeof oldMatch.winner_id === 'number' &&
    oldParticipantIdMap.has(oldMatch.winner_id)
  ) {
    const winnerName = oldParticipantIdMap.get(
      oldMatch.winner_id as number,
    ).name;
    winner_id = newParticipantNameMap.get(winnerName).id;
    console.log(
      `Migrating match ${currentMatchPlayer1Name} vs ${currentMatchPlayer2Name} with winner ${winnerName}.`,
    );
  } else {
    winner_id = oldMatch.winner_id;
    if (oldMatch.scores_csv === '0-0') {
      winner_id = 'tie';
    }
    console.log(
      `Migrating match ${currentMatchPlayer1Name} vs ${currentMatchPlayer2Name} with result ${winner_id}.`,
    );
  }
  const oldPlayerIds = [oldMatch.player1_id, oldMatch.player2_id];
  await Promise.all(oldPlayerIds.map((pid) => checkQuit(match.round, pid)));
  while (true) {
    try {
      await axios.put(
        `https://api.challonge.com/v1/tournaments/${config.targetTournament}/matches/${match.id}.json`,
        {
          api_key: config.apiKey,
          match: {
            scores_csv: oldMatch.scores_csv,
            winner_id,
          },
        },
        axiosPostConfig,
      );
      console.log(
        `Successfully migrated match ${currentMatchPlayer1Name} vs ${currentMatchPlayer2Name}.`,
      );
      break;
    } catch (e) {
      console.error(
        `Failed migrating match ${currentMatchPlayer1Name} vs ${currentMatchPlayer2Name}: ${e.toString()}`,
      );
    }
  }
}

async function main() {
  config = await loadConfig();
  console.log(`Reading old tournament ${config.sourceTournament}.`);
  await loadOldTournamentData();
  for (let round = 1; round <= config.maxRounds; ++round) {
    console.log(`Handling round ${round}.`);
    await loadNewTournamentData();
    const matches = newMatches.filter(
      (m) => m.round === round && m.state === 'open',
    );
    if (!matches.length) {
      console.log(`No available matches in round ${round}.`);
      continue;
    }
    await Promise.all(matches.map((match) => migrateMatch(match)));
  }
  console.log(`Finished.`);
}
main();
